@@ -14,7 +14,9 @@ pub enum SQLOperator {
     NotEq,
     And,
     Or,
+    Not,
     Like,
+    NotLike,
 }
 
 impl ToString for SQLOperator {
@@ -33,7 +35,9 @@ impl ToString for SQLOperator {
             SQLOperator::NotEq => "!=".to_string(),
             SQLOperator::And => "AND".to_string(),
             SQLOperator::Or => "OR".to_string(),
+            SQLOperator::Not => "NOT".to_string(),
             SQLOperator::Like => "LIKE".to_string(),
+            SQLOperator::NotLike => "NOT LIKE".to_string(),
         }
     }
 }
